<?php

namespace Modules\Cotacao\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\Report\ReportService;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DataSource\DataSet;
use Modules\Cotacao\Services\PedidoColetaService;
use Modules\Cotacao\Modules\Mercadum\Controllers\IntegracaoMercadumController;
use PhpOffice\PhpSpreadsheet\Spreadsheet;
use PhpOffice\PhpSpreadsheet\Writer\Xlsx;
use PhpOffice\PhpSpreadsheet\Style\Protection;
use ZipArchive;

class PedidoColetaController extends \Zeedhi\Framework\Controller\Report
{

    protected $entityManager;
    protected $smtpUtil;
    protected $supUtilities;
    protected $reportService;
    protected $qr2Strategy;
    protected $environment;
    protected $service;
    protected $birtStrategy;
    protected $integracaoMercadum;
    protected $newCode;
    protected $crudService;
    protected $tmpPath = '/tmp/';

    public function __construct($reportService, $qr2Strategy, $entityManager, $smtpUtil, $supUtilities, PedidoColetaService $service, $environment, $newCode, $connection, $birtStrategy, IntegracaoMercadumController $integracaoMercadum, $crudService)
    {
        $this->entityManager = $entityManager;
        $this->smtpUtil = $smtpUtil;
        $this->supUtilities = $supUtilities;
        $this->reportService = $reportService;
        $this->qr2Strategy = $qr2Strategy;
        $this->service = $service;
        $this->environment = $environment;
        $this->connection = $connection;
        $this->newCode = $newCode;
        $this->birtStrategy = $birtStrategy;
        $this->integracaoMercadum = $integracaoMercadum;
        $this->crudService = $crudService;
    }

    public function getDataPedidoColeta(DTO\Request\Row $request, DTO\Response $response)
    {
        try {
            $row = $request->getRow();

            $comprador = $this->verifyComprador($row);
            $datasetEmail = [];
            $exists = [];

            if (empty($comprador['message']) && empty($comprador['comprador'])) {
                $connection = $this->entityManager->getConnection();
                $procedure = new StoredProcedure($connection, 'PEDIDO_COLETA.GETDATAPEDIDOCOLETA');

                $procedure->addParam(new Param('P_CDFILIAL',     Param::PARAM_INPUT, $row['CDFILIAL']));
                $procedure->addParam(new Param('P_CDFILIPROC',   Param::PARAM_INPUT, $this->environment->getCdFilial()));
                $procedure->addParam(new Param('P_CDCOMPRADOR',  Param::PARAM_INPUT, $row['CDCOMPRADOR']));
                $procedure->addParam(new Param('P_CDPRODINI',    Param::PARAM_INPUT, $row['CDPRODINI']));
                $procedure->addParam(new Param('P_CDPRODFIN',    Param::PARAM_INPUT, $row['CDPRODFIN']));
                $procedure->addParam(new Param('P_CDFORNECED',   Param::PARAM_INPUT, $row['CDFORNECED']));
                $procedure->addParam(new Param('P_CDREGIAO',     Param::PARAM_INPUT, $row['CDREGIAO']));
                $procedure->addParam(new Param('P_CDMARCA',      Param::PARAM_INPUT, $row['CDMARCA']));
                $procedure->addParam(new Param('P_DTINI',        Param::PARAM_INPUT, $row['DTINI']));
                $procedure->addParam(new Param('P_DTFIN',        Param::PARAM_INPUT, $row['DTFIN']));
                $procedure->addParam(new Param('P_TIPOPEDIDO',   Param::PARAM_INPUT, $row['TIPOPEDIDO']));
                $procedure->addParam(new Param('P_NRCOLEPRECO',  Param::PARAM_INPUT, $row['NRCOLEPRECO']));
                $procedure->addParam(new Param('P_DESCFORNCOT',  Param::PARAM_INPUT, $row['DESCFORNCOT']));
                $procedure->addParam(new Param('P_NRCOMPRA',     Param::PARAM_INPUT, $row['NRCOMPRA']));
                $procedure->addParam(new Param('P_CDLEILAO',     Param::PARAM_INPUT, $row['CDLEILAO']));
                $procedure->addParam(new Param('P_NRSEQPRELIST', Param::PARAM_INPUT, $row['NRSEQPRELIST']));
                $procedure->addParam(new Param('P_CDOPERLANC',   Param::PARAM_INPUT, $this->environment->getCdoperador()));
                $procedure->addParam(new Param('P_NRORG',        Param::PARAM_INPUT, $this->environment->getNrOrg()));
                $procedure->addParam(new Param('P_NRPEDCOL',     Param::PARAM_OUTPUT, NULL, Param::PARAM_TYPE_STR, 20));
                $procedure->addParam(new Param('P_ERROR',        Param::PARAM_OUTPUT, NULL, Param::PARAM_TYPE_STR, 20));
                $result = $procedure->execute();

                $datasetEmail = $this->getFornecedorEmail($result);
                $exists = $this->existePedidoColeta($result);

            }

            $response->addDataSet(new DataSet("data", array('pedidos' => $datasetEmail, 'exists' => $exists, 'comprador' => $comprador)));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), 1));
        }
    }

    private function verifyComprador($row)
    {
        try {
            $cdoperador = $this->environment->getCdoperador();

            $operadorRelac = $this->getOperadorRelacionado($row);
            $relacionado = array_filter($operadorRelac, function ($oper) use ($cdoperador) {
                return $oper['CDOPERADOR'] == $cdoperador;
            });

            $message = empty($relacionado) ? 'NO_OPERATOR_RELATED_BUYER' : '';

            $compradorNaoRelacionado = array();
            if (empty($message)) {
                $compradorNaoRelacionado = $this->getOperadorNaoRelacionado($row, $cdoperador);
            }

            return array('message' => $message, 'comprador' => $compradorNaoRelacionado);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    private function getOperadorRelacionado($param)
    {
        try {
            $conditions = array(
                array(
                    'columnName' => 'P_CDCOMPRADOR',
                    'operator' => '=',
                    "value" => $param['CDCOMPRADOR']
                )
            );

            return $this->crudService->getFilteredDataSet("operadorRelacionado", $conditions)->getRows();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    private function getOperadorNaoRelacionado($param, $cdoperador)
    {
        try {
            $conditions = array(
                array(
                    'columnName' => 'P_CDCOMPRADOR',
                    'operator' => '=',
                    "value" => $param['CDCOMPRADOR']
                ),
                array(
                    'columnName' => 'P_CDOPERADOR',
                    'operator' => '=',
                    "value" => $cdoperador
                )
            );

            return $this->crudService->getFilteredDataSet("operadorNaoRelacionado", $conditions)->getRows();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    private function getFornecedorEmail($param)
    {
        try {
            $conditions = array(
                array(
                    'columnName' => 'P_NRPEDCOL',
                    'operator' => '=',
                    "value" => $param['P_NRPEDCOL']
                )
            );

            return $this->crudService->getFilteredDataSet("fornecedorEmail", $conditions)->getRows();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    private function existePedidoColeta($param)
    {
        try {
            $conditions = array(
                array(
                    'columnName' => 'P_NRPEDCOL',
                    'operator' => '=',
                    "value" => $param['P_NRPEDCOL']
                )
            );

            return $this->crudService->getFilteredDataSet("existePedidoColeta", $conditions)->getRows();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function getMensagemEnvioEmail(DTO\Request\Row $request, DTO\Response $response)
    {
        try {
            $msgManual = $this->supUtilities->getMensagemPadraoEmail('S');
            $msgPortal = $this->supUtilities->getMensagemPadraoEmail('P');
            $msgArquivo = $this->supUtilities->getMensagemPadraoEmail('A');
            $response->addDataSet(new DataSet("data", array('manual' => $msgManual, 'portal' => $msgPortal, 'email' => $msgArquivo)));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), 500));
        }
    }

    private function pedcolPrevcons($row, $typeOutput)
    {
        try {
            $connection = $this->entityManager->getConnection();
            $procedure = new StoredProcedure($connection, 'PEDIDO_COLETA.SAVEPEDCOL_PREVCONS');

            $procedure->addParam(new Param('P_CDFILIAL',  Param::PARAM_INPUT,  $typeOutput === 'MAIL' ? $row['CDFILIAL'] : $row['P_CDFILIAL']));
            $procedure->addParam(new Param('P_PRODINI',   Param::PARAM_INPUT,  $typeOutput === 'MAIL' ? $row['PRODINI'] : $row['P_CDPRODINI']));
            $procedure->addParam(new Param('P_PRODFIN',   Param::PARAM_INPUT,  $typeOutput === 'MAIL' ? $row['PRODFIN'] : $row['P_CDPRODFIN']));
            $procedure->addParam(new Param('P_MARCA',     Param::PARAM_INPUT,  $typeOutput === 'MAIL' ? $row['MARCA'] : $row['P_CDMARCA']));
            $procedure->addParam(new Param('P_COMPRADOR', Param::PARAM_INPUT,  $typeOutput === 'MAIL' ? $row['COMPRADOR'] : $row['P_CDCOMPRADOR']));
            $procedure->addParam(new Param('P_REGIAO',    Param::PARAM_INPUT,  $typeOutput === 'MAIL' ? $row['REGIAO'] : $row['P_CDREGIAO']));
            $procedure->addParam(new Param('P_FORNECED',  Param::PARAM_INPUT,  $typeOutput === 'MAIL' ? $row['FORNECED'] : $row['P_CDFORNECED']));
            $procedure->addParam(new Param('P_DTINI',     Param::PARAM_INPUT,  $typeOutput === 'MAIL' ? $row['DTINI'] : $row['P_DTINI']));
            $procedure->addParam(new Param('P_DTFIN',     Param::PARAM_INPUT,  $typeOutput === 'MAIL' ? $row['DTFIN'] : $row['P_DTFIN']));
            $procedure->addParam(new Param('P_NRORG',     Param::PARAM_INPUT,  $this->environment->getNrOrg()));
            $procedure->addParam(new Param('P_NRPEDCOL',  Param::PARAM_INPUT,  $typeOutput === 'MAIL' ? $row['NRPEDCOL'] : $row['P_NRPEDCOL']));

            $procedure->execute();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    private function savePedidoColeta($row)
    {
        try {

            $connection = $this->entityManager->getConnection();
            $procedure = new StoredProcedure($connection, 'PEDIDO_COLETA.SAVEPEDIDOCOLETA');

            $procedure->addParam(new Param('P_NRPEDCOL',       Param::PARAM_INPUT,  $row['NRPEDCOL']));
            $procedure->addParam(new Param('P_CDFILIAL',       Param::PARAM_INPUT,  $row['CDFILIAL']));
            $procedure->addParam(new Param('P_CDFILIPROC',     Param::PARAM_INPUT,  $this->environment->getCdFilial()));
            $procedure->addParam(new Param('P_QTMESES',        Param::PARAM_INPUT,  $row['QTMESES']));
            $procedure->addParam(new Param('P_DTEXPIRAPEDCOL', Param::PARAM_INPUT,  $row['DTEXPIRAPEDCOL']));
            $procedure->addParam(new Param('P_IDPEDGERALREG',  Param::PARAM_INPUT,  $row['IDPEDGERALREG']));
            $procedure->addParam(new Param('P_IDPEDGERALFORN', Param::PARAM_INPUT,  $row['IDPEDGERALFORN']));
            $procedure->addParam(new Param('P_IDCURVAABCAF',   Param::PARAM_INPUT,  $row['IDCURVAABCAF']));
            $procedure->addParam(new Param('P_IDCONSFORNAF',   Param::PARAM_INPUT,  $row['IDCONSFORNAF']));
            $procedure->addParam(new Param('P_ERROR',          Param::PARAM_OUTPUT, NULL, Param::PARAM_TYPE_STR,  10));

            $procedure->execute();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    private function getParamMercadum()
    {
        try {
            return $this->crudService->getFilteredDataSet("parammercadum", [])->getRows();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function savePedidoColetaWeb(DTO\Request\Row $request, DTO\Response $response)
    {
        $row = $request->getRow();
        try {

            if ($row['BOACRESITEM'] == 'S') {
                $this->pedcolPrevcons($row, 'MAIL');
            }

            $this->savePedidoColeta($row);

            if ($row['SEND_EMAIL'] == 'S') {
                $pedidos = $this->sendEmailPedidoColetaWeb($row);
            } else {
                $pedidos = [];
            }

            /*
             * Retirar esse comentário e verificar o controller resposável pela integração
             * com o mercadum
             *

            $parammercadum = $this->getParamMercadum();

            if (isset($parammercadum) && $parammercadum[0]['IDINTGPEDCOL'] == 'S') {
                $this->integracaoMercadum->integracaoMercadumPedidoColeta($row);
            }*/

            $response->addDataSet(new DataSet('pedidos', $pedidos));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), 500));
        }
    }

    private function makeHtmlMessage($subject, $sentBy, $message)
    {
        return "<table>
                    <tr>
                        <td>"
            . $subject . "
                        </td>
                    </tr>

                    <tr>
                        <td class='description'>
                            <br/>" . $message .
            "</td>
                    </tr>

                    <tr class='footer'>
                        <td>
                            <center>
                                <br> <p>" . $sentBy . " </p>
                            </center>
                        </td>
                    </tr>
                </table>";
    }

    private function sendEmailPedidoColetaWeb($row)
    {
        if (isset($row['REQUESTS']) && is_array($row['REQUESTS'])) {
            foreach ($row['REQUESTS'] as &$request) {
                if (isset($request['DSEMAILCOT']) && is_array($request['DSEMAILCOT']) && count($request['DSEMAILCOT'])) {
                    $message = $this->makeHtmlMessage($row['SUBJECT'], $row['SENT_BY'], $request['EMAIL_MESSAGE']);
                    $messageSendMail = $this->supUtilities->messageSendMail($this->environment->getCdFilial(), $row['SUBJECT'], $message, 'PORTAL E-COMMERCE');
                    try {
                        $result = $this->supUtilities->enviaEmail($row["LOGGED_EMAIL"], $row['SUBJECT'], $row["LOGGED_EMAIL"], $messageSendMail, $request['DSEMAILCOT']);

                        $request['RESULT'] = 'SUCCESS';
                    } catch (\Exception $e) {
                        $request['RESULT'] = 'ERROR';
                        $request['ERROR'] = $e->getMessage();
                    }
                } else {
                    $request['RESULT'] = 'ERROR';
                    $request['ERROR'] = 'NOTMAILREGISTER';
                }
            }
        }

        return $row['REQUESTS'];
    }

    private function getPedColExporta($nrpedcol)
    {
        try {
            $conditions = array(
                array(
                    'columnName' => 'P_NRPEDCOL',
                    'operator' => '=',
                    "value" => $nrpedcol
                )
            );

            return $this->crudService->getFilteredDataSet("getPedColExporta", $conditions)->getRows();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    private function createZip($zipName, $fileName, $zipped)
    {
        try {
            if ($zipped == 'S') {
                $zip = new ZipArchive();

                foreach ($fileName as $file) {
                    if ($zip->open($this->tmpPath . $zipName, ZipArchive::CREATE) === TRUE) {
                        if ($handle = opendir($this->tmpPath)) {
                            $zip->addFile($this->tmpPath . $file . '.xlsx', $file . '.xlsx');
                            closedir($handle);
                        }
                        $zip->close();
                        unlink($this->tmpPath . $file . '.xlsx');
                    }
                }
                $dataArchive = array(
                    'NOMEARQUIVOZIP'    => $zipName,
                    'CAMINHOARQUIVOZIP' => $this->tmpPath,
                );
            } else if ($zipped == 'N') {
                $fileName = "" . implode(', ', $fileName);
                $xlsPath = $this->tmpPath . $fileName . '.xlsx';
                $dataArchive = array(
                    'NOMEARQUIVOXLS'    => $fileName . '.xlsx',
                    'CAMINHOARQUIVOXLS' => $xlsPath,
                );
            }
            return $dataArchive;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    private function geraXlsxPedCol($rowsXlsx, $comprador, $fornecedor, $nameZipArchive)
    {
        try {
            $spreadsheet = new Spreadsheet();
            $activeWorksheet = $spreadsheet->getActiveSheet();
            $header = array(
                'REGIAO',
                'NOME_REGIAO',
                'VALIDADE_INI',
                'VALIDADE_FIN',
                'COMPRADOR',
                'NOME_COMPRADOR',
                'FORNECEDOR',
                'NM_FORNECEDOR',
                'PRODUTO',
                'NOME_PR',
                'MARCA',
                'NOME_MARCA',
                'QTDE',
                'VRUNI',
                'OBS_FORNECEDOR'
            );
            $styleArray = [
                'font' => [
                    'bold' => true
                ]
            ];
            $spreadsheet->getDefaultStyle()->getFont()->setName('Arial');
            $spreadsheet->getDefaultStyle()->getFont()->setSize(10);
            $activeWorksheet->getProtection()->setPassword('TeknisaSup@2024');
            $activeWorksheet->getProtection()->setSheet(true);
            $activeWorksheet->fromArray($header, NULL, 'A1');
            $spreadsheet->getActiveSheet()->getStyle('A1:O1')->applyFromArray($styleArray);

            foreach (range('A', 'L') as $key => $value) {
                $spreadsheet->getActiveSheet()->getColumnDimension($value)->setAutoSize(true);
            }

            $indice = 2;
            foreach ($rowsXlsx as $row) {
                $cell = 'A' . $indice;
                $cellQtde = 'M' . $indice;
                $cellValorUni = 'N' . $indice;
                $cellObs = 'O' . $indice;
                $activeWorksheet->fromArray($row, NULL, $cell);
                $activeWorksheet->getStyle($cellQtde)->getProtection()->setLocked(Protection::PROTECTION_UNPROTECTED);
                $activeWorksheet->getStyle($cellValorUni)->getProtection()->setLocked(Protection::PROTECTION_UNPROTECTED);
                $activeWorksheet->getStyle($cellObs)->getProtection()->setLocked(Protection::PROTECTION_UNPROTECTED);
                $indice++;
            }
            $nameArchive = date("Y-m-d_h-i-s") . '_' .  $comprador . '_' . $fornecedor;
            $writer = new Xlsx($spreadsheet);
            $writer->save($this->tmpPath . $nameArchive . '.xlsx');

            return array(
                'ZIPFILENAME' => $nameZipArchive,
                'FILENAME'    => $nameArchive
            );
        } catch (\Exception $e) {
            throw $e;
        }
    }

    private function montaDadosXlsx($dataSet)
    {
        $rowsXlsx = array();
        foreach ($dataSet as $row) {

            $data = array(
                $row['CDREGIAO'],
                $row['NMREGIAO'],
                $row['VALIDADE_INI'],
                $row['VALIDADE_FIN'],
                $row['CDOPERADOR'],
                $row['NMOPERADOR'],
                '_' . $row['NRINSJURFORN'],
                $row['NMRAZSOCFORN'],
                '_' . $row['CDARVPROD'],
                $row['NMPRODUTO'],
                $row['CDMARCA'],
                $row['NMMARCA'],
                $row['QTACOMPRAR']
            );
            array_push($rowsXlsx, $data);
        };

        return $rowsXlsx;
    }

    public function exportaXlsx(DTO\Request $request, DTO\Response $response)
    {
        try {
            $row = $request->getRow();
            $dataset = $this->getPedColExporta($row['NRPEDCOL']);
            $requests = $row['REQUESTS'];

            $suppliers = array_unique(array_map(function ($row) {
                return $row['CDFORNECED'];
            }, $dataset));

            $nameZipArchive = "PEDCOL" . uniqid() . ".zip";
            $fileName = array();
            $downloadfileName = array();
            $result = array();
            foreach ($suppliers as &$supplier) {
                $emailData = array_filter($requests, function ($row) use ($supplier) {
                    return $row['CDFORNECED'] == $supplier;
                });

                $supplierData = array_filter($dataset, function ($row) use ($supplier) {
                    return $row['CDFORNECED'] == $supplier;
                });

                $supplierFileName = isset($supplierData) && count($supplierData) ? end($supplierData)["NRINSJURFORN"] . end($supplierData)["NMFANTFORN"] : "";
                $replace = array("/", ".");
                $supplierFileName = str_replace($replace, "", $supplierFileName);

                $buyers = array_unique(array_map(function ($row) {
                    return $row['CDOPERADOR'];
                }, $supplierData));

                foreach ($buyers as $buyer) {
                    $buyerData = array_filter($supplierData, function ($row) use ($buyer) {
                        return $row['CDOPERADOR'] == $buyer;
                    });
                    $rowsXlsx = $this->montaDadosXlsx($buyerData);

                    if (($row['EMAILORFILE'] === 'E') && ($row["checkboxXLSEMAIL"] == 'S')) {
                        $nameZipArchive = "PEDCOL" . uniqid() . ".zip";

                        $xls = $this->geraXlsxPedCol($rowsXlsx, $buyer, $supplierFileName, $nameZipArchive);
                        $zipName = $xls['ZIPFILENAME'];
                        $fileName = $xls['FILENAME'];
                        $nameReturn = $this->createZip($zipName, [$fileName], $row["checkboxXLSEMAIL"]);

                        foreach ($emailData as &$data) {
                            if (isset($data['DSEMAILCOT']) && is_array($data['DSEMAILCOT']) && count($data['DSEMAILCOT'])) {
                                $message = $this->makeHtmlMessage($row['SUBJECT'], $row['SENT_BY'], $data['EMAIL_MESSAGE']);
                                $messageSendMail = $this->supUtilities->messageSendMail($this->environment->getCdFilial(), $row['SUBJECT'], $message, 'PORTAL E-COMMERCE');
                                try {
                                    $this->supUtilities->enviaEmail($row["LOGGED_EMAIL"], $row['SUBJECT'], $row["LOGGED_EMAIL"], $messageSendMail, $data['DSEMAILCOT'], $nameReturn["CAMINHOARQUIVOZIP"] . $nameReturn["NOMEARQUIVOZIP"], $nameReturn["NOMEARQUIVOZIP"]);
                                    unlink($nameReturn["CAMINHOARQUIVOZIP"] . $nameReturn["NOMEARQUIVOZIP"]);

                                    $data['RESULT'] = 'SUCCESS';
                                } catch (\Exception $e) {
                                    $data['RESULT'] = 'ERROR';
                                    $data['ERROR'] = $e->getMessage();
                                }
                            } else {
                                $data['RESULT'] = 'ERROR_EMAIL';
                                $data['ERROR_EMAIL'] = 'NOTMAILREGISTER';
                            }
                        }

                        $result = array_merge($result, $emailData);
                    } else if (($row['EMAILORFILE'] === 'E') && ($row["checkboxXLSEMAIL"] == 'N')) {
                        $nameZipArchive = "PEDCOL" . uniqid();
                        $xls = $this->geraXlsxPedCol($rowsXlsx, $buyer, $supplierFileName, $nameZipArchive);
                        $zipName = $xls['ZIPFILENAME'];
                        $fileName = $xls['FILENAME'];
                        $nameReturn = $this->createZip($zipName, [$fileName], $row["checkboxXLSEMAIL"]);

                        foreach ($emailData as &$data) {
                            if (isset($data['DSEMAILCOT']) && is_array($data['DSEMAILCOT']) && count($data['DSEMAILCOT'])) {
                                $message = $this->makeHtmlMessage($row['SUBJECT'], $row['SENT_BY'], $data['EMAIL_MESSAGE']);
                                $messageSendMail = $this->supUtilities->messageSendMail($this->environment->getCdFilial(), $row['SUBJECT'], $message, 'PORTAL E-COMMERCE');
                                try {
                                    $this->supUtilities->enviaEmail($row["LOGGED_EMAIL"], $row['SUBJECT'], $row["LOGGED_EMAIL"], $messageSendMail, $data['DSEMAILCOT'], $nameReturn["CAMINHOARQUIVOXLS"], $nameReturn["NOMEARQUIVOXLS"]);
                                    $data['RESULT'] = 'SUCCESS';
                                } catch (\Exception $e) {
                                    $data['RESULT'] = 'ERROR';
                                    $data['ERROR'] = $e->getMessage();
                                }
                            } else {
                                $data['RESULT'] = 'ERROR_EMAIL';
                                $data['ERROR_EMAIL'] = 'NOTMAILREGISTER';
                            }
                        }
                        $result = array_merge($result, $emailData);
                    } else {
                        $xls = $this->geraXlsxPedCol($rowsXlsx, $buyer, $supplierFileName, $nameZipArchive);
                        $zipName = $xls['ZIPFILENAME'];
                        $fileName = $xls['FILENAME'];
                        array_push($downloadfileName, $fileName);
                    }
                }
            };

            if ($row['EMAILORFILE'] === 'D') {
                $result = $this->createZip($zipName, $downloadfileName, 'S');
            }

            $this->service->deletePedidoColetaTmp($row['NRPEDCOL']);
            $response->addDataSet(new DataSet('returnFilePath', $result));
        } catch (\Exception $e) {
            $this->service->deletePedidoColetaTmp($row['NRPEDCOL']);
            $response->setError(new DTO\Response\Error($e->getMessage(), 1));
        }
    }

    public function gerarPedidoColeta(DTO\Request $request, DTO\Response $response)
    {
        $row = $request->getRow();
        $dataset = $this->getPedColExporta($row['NRPEDCOL']);

        $result = array();

        $filterDataDownload = array_filter($dataset, function ($row) {
            return $row['IDTPAUTPEDCOL'] == "DOWNLOAD";
        });

        $filterDataEmail = array_filter($dataset, function ($row) {
            return $row['IDTPAUTPEDCOL'] == "EMAIL";
        });

        $filterDataPortal = array_filter($dataset, function ($row) {
            return $row['IDTPAUTPEDCOL'] == "PORTAL";
        });

        foreach ($filterDataEmail as &$email) {
            foreach ($row['REQUESTS'] as $request) {
                if ($email['CDFORNECED']    === $request['CDFORNECED']) {
                    $email['DSEMAILCOT']    = $request['DSEMAILCOT'];
                    $email['EMAIL_MESSAGE'] = $request['EMAIL_MESSAGE'];
                }
            }
        }

        if (count($filterDataEmail)) {
            $result = $this->enviaPlanilhaEmail($filterDataEmail, $row);
            $response->addDataSet(new DataSet('returnXlsxEmail', $result));
        }
        if (count($filterDataPortal)) {
            try{
                $pedidos = $this->enviaPedidoEcommerce($filterDataPortal, $row);
                $response->addDataSet(new DataSet('pedidos', $pedidos));
            } catch (\Exception $e) {
                $response->setError(new DTO\Response\Error($e->getMessage(), 500));
            }
        }
        if (count($filterDataDownload)) {
            $result = $this->downloadPlanilha($filterDataDownload, $dataset);
            $response->addDataSet(new DataSet('returnFilePath', $result));
        }
    }

    private function pedcolPrevconsAutomatico($row, $typeOutput)
    {
        try {
            $connection = $this->entityManager->getConnection();
            $procedure = new StoredProcedure($connection, 'PEDIDO_COLETA.SAVEPEDCOL_PREVCONS');

            $procedure->addParam(new Param('P_CDFILIAL',  Param::PARAM_INPUT,  $typeOutput === 'MAIL' ? $row['CDFILIAL'] : $row['CDFILIAL']));
            $procedure->addParam(new Param('P_PRODINI',   Param::PARAM_INPUT,  $typeOutput === 'MAIL' ? $row['CDPRODINI'] : $row['CDPRODINI']));
            $procedure->addParam(new Param('P_PRODFIN',   Param::PARAM_INPUT,  $typeOutput === 'MAIL' ? $row['CDPRODFIN'] : $row['CDPRODFIN']));
            $procedure->addParam(new Param('P_MARCA',     Param::PARAM_INPUT,  $typeOutput === 'MAIL' ? $row['MARCA'][0] : $row['MARCA'][0]));
            $procedure->addParam(new Param('P_COMPRADOR', Param::PARAM_INPUT,  $typeOutput === 'MAIL' ? $row['COMPRADOR'][0] : $row['COMPRADOR'][0]));
            $procedure->addParam(new Param('P_REGIAO',    Param::PARAM_INPUT,  $typeOutput === 'MAIL' ? $row['REGIAO'][0] : $row['REGIAO'][0]));
            $procedure->addParam(new Param('P_FORNECED',  Param::PARAM_INPUT,  $typeOutput === 'MAIL' ? $row['FORNECEDOR'][0] : $row['FORNECEDOR'][0]));
            $procedure->addParam(new Param('P_DTINI',     Param::PARAM_INPUT,  $typeOutput === 'MAIL' ? $row['INTERVAL_VALIDADE']['start'] : $row['INTERVAL_VALIDADE']['start']));
            $procedure->addParam(new Param('P_DTFIN',     Param::PARAM_INPUT,  $typeOutput === 'MAIL' ? $row['INTERVAL_VALIDADE']['end'] : $row['INTERVAL_VALIDADE']['end']));
            $procedure->addParam(new Param('P_NRORG',     Param::PARAM_INPUT,  $this->environment->getNrOrg()));
            $procedure->addParam(new Param('P_NRPEDCOL',  Param::PARAM_INPUT,  $typeOutput === 'MAIL' ? $row['NRPEDCOL'] : $row['NRPEDCOL']));

            $procedure->execute();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    private function enviaPedidoEcommerce($filterDataPortal, $row)
    {
        if ($row['BOACRESITEM'] == 'S') {
            $this->pedcolPrevconsAutomatico($row['ROW_PEDIDO'], 'MAIL');
        }
        $row['CDFILIAL'] = $row['ROW_PEDIDO']['CDFILIAL'];

        $this->savePedidoColeta($row);

        if ($row['SEND_EMAIL'] == 'S') {

            $pedidos = $this->sendEmailPedidoColetaWebAutomatico($row, $filterDataPortal);
        } else {
            $pedidos = [];
        }

        return $pedidos;
    }

    private function sendEmailPedidoColetaWebAutomatico($row, $filterDataPortal)
    {
        if (isset($row['REQUESTS']) && is_array($row['REQUESTS'])) {
            foreach ($row['REQUESTS'] as &$request) {
                if (isset($request['DSEMAILCOT']) && is_array($request['DSEMAILCOT']) && count($request['DSEMAILCOT']) && $request['IDTPAUTPEDCOL'] == 'PORTAL') {
                    $message = $this->makeHtmlMessage($row['SUBJECTPORTAL'], $row['SENT_BY'], $request['EMAIL_MESSAGE']);
                    $messageSendMail = $this->supUtilities->messageSendMail($this->environment->getCdFilial(), $row['SUBJECTPORTAL'], $message, 'PORTAL E-COMMERCE');
                    try {
                        $result = $this->supUtilities->enviaEmail($row["LOGGED_EMAIL"], $row['SUBJECTPORTAL'], $row["LOGGED_EMAIL"], $messageSendMail, $request['DSEMAILCOT']);

                        $request['RESULT'] = 'SUCCESS';
                    } catch (\Exception $e) {
                        $request['RESULT'] = 'ERROR';
                        $request['ERROR'] = $e->getMessage();
                    }
                } else {
                    $request['RESULT'] = 'ERROR';
                    $request['ERROR'] = 'NOTMAILREGISTER';
                }
            }
        }

        return $row['REQUESTS'];
    }

    private function enviaPlanilhaEmail($filterDataEmail, $frontendRow)
    {
        $result = array();
        $emailData = array();

        $buyers = array_unique(array_map(function ($row) {
            return $row['CDOPERADOR'];
        }, $filterDataEmail));

        $nameZipArchive = "PEDCOL" . uniqid() . ".zip";

        $suppliers = array_unique(array_map(function ($row) {
            return $row['CDFORNECED'];
        }, $filterDataEmail));

        foreach ($suppliers as &$supplier) {
            $emailData = array_filter($frontendRow['REQUESTS'], function ($row) use ($supplier) {
                return $row['CDFORNECED'] == $supplier;
            });

            $supplierData = array_filter($filterDataEmail, function ($row) use ($supplier) {
                return $row['CDFORNECED'] == $supplier;
            });

            $supplierFileName = isset($supplierData) && count($supplierData) ? end($supplierData)["NRINSJURFORN"] . end($supplierData)["NMFANTFORN"] : "";
            $replace = array("/", ".");
            $supplierFileName = str_replace($replace, "", $supplierFileName);

            foreach ($buyers as $buyer) {
                $buyerData = array_filter($supplierData, function ($row) use ($buyer) {
                    return $row['CDOPERADOR'] == $buyer;
                });
                $rowsXlsx = $this->montaDadosXlsx($buyerData);

                $xls = $this->geraXlsxPedCol($rowsXlsx, $buyer, $supplierFileName, $nameZipArchive);
                $zipName = $xls['ZIPFILENAME'];
                $fileName = $xls['FILENAME'];
                $nameReturn = $this->createZip($zipName, [$fileName], 'S');

                foreach ($emailData as &$data) {
                    if (isset($data['DSEMAILCOT']) && is_array($data['DSEMAILCOT']) && count($data['DSEMAILCOT']) && $data['IDTPAUTPEDCOL'] == 'EMAIL') {
                        $message = $this->makeHtmlMessage($frontendRow['SUBJECTEMAIL'], $frontendRow['SENT_BY'], $data['EMAIL_MESSAGE']);
                        $messageSendMail = $this->supUtilities->messageSendMail($this->environment->getCdFilial(), $frontendRow['SUBJECTEMAIL'], $message, 'PORTAL E-COMMERCE');
                        try {
                            $this->supUtilities->enviaEmail($frontendRow["LOGGED_EMAIL"], $frontendRow['SUBJECTEMAIL'], $frontendRow["LOGGED_EMAIL"], $messageSendMail, $data['DSEMAILCOT'], $nameReturn["CAMINHOARQUIVOZIP"] . $nameReturn["NOMEARQUIVOZIP"], $nameReturn["NOMEARQUIVOZIP"]);
                            unlink($nameReturn["CAMINHOARQUIVOZIP"] . $nameReturn["NOMEARQUIVOZIP"]);

                            $data['RESULT'] = 'SUCCESS';
                        } catch (\Exception $e) {
                            $data['RESULT'] = 'ERROR';
                            $data['ERROR'] = $e->getMessage();
                        }
                    } else {
                        $data['RESULT'] = 'ERROR_EMAIL';
                        $data['ERROR_EMAIL'] = 'NOTMAILREGISTER';
                    }
                }

                $result = array_merge($result, $supplierData);
            }
        }

        return $result;
    }

    private function downloadPlanilha($filterDataDownload, $dataset)
    {
        $downloadfileName = array();
        $zipName = '';
        $fileName = '';
        $buyers = array_unique(array_map(function ($row) {
            return $row['CDOPERADOR'];
        }, $dataset));

        $nameZipArchive = "PEDCOL" . uniqid() . ".zip";

        $suppliers = array_unique(array_map(function ($row) {
            return $row['CDFORNECED'];
        }, $filterDataDownload));

        foreach ($suppliers as &$supplier) {
            $supplierData = array_filter($filterDataDownload, function ($row) use ($supplier) {
                return $row['CDFORNECED'] == $supplier;
            });

            $supplierFileName = isset($supplierData) && count($supplierData) ? end($supplierData)["NRINSJURFORN"] . end($supplierData)["NMFANTFORN"] : "";
            $replace = array("/", ".");
            $supplierFileName = str_replace($replace, "", $supplierFileName);
            foreach ($buyers as $buyer) {
                $buyerData = array_filter($supplierData, function ($row) use ($buyer) {
                    return $row['CDOPERADOR'] == $buyer;
                });
                $rowsXlsx = $this->montaDadosXlsx($buyerData);

                $xls = $this->geraXlsxPedCol($rowsXlsx, $buyer, $supplierFileName, $nameZipArchive);
                $zipName = $xls['ZIPFILENAME'];
                $fileName = $xls['FILENAME'];
                array_push($downloadfileName, $fileName);
            }
        };

        $result = $this->createZip($zipName, $downloadfileName, 'S');
        $result['DOWNLOAD'] = 'S';

        $this->service->deletePedidoColetaTmp($dataset[0]['NRPEDCOL']);

        return $result;
    }

    public function downloadZipPedCol(DTO\Request $request, DTO\Response $response)
    {
        $row = $request->getDataSet()->getRows()[0];
        $file = $row["CAMINHOARQUIVOZIP"];
        $name = $row["NOMEARQUIVOZIP"];
        $nameArray = explode(".", $name);
        header('Accept-Ranges: none');
        header("Content-Type: application/zip");
        header("Content-Disposition: attachment; filename=" . $name);
        readfile($file . $name);
        unlink($row['CAMINHOARQUIVOZIP'] . $row['NOMEARQUIVOZIP']);
        exit;
    }

    public function getInitialParams(DTO\Request\Filter $request, DTO\Response $response)
    {
        try {
            $conditions = array(
                array(
                    'columnName' => 'CDFILIAL',
                    'operator' => '=',
                    "value" => $this->environment->getCdFilial()
                )
            );

            $dataSet = $this->crudService->getFilteredDataSet("data_pedido_coleta", $conditions)->getRows();

            $response->addDataSet(new DataSet('data', $dataSet));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    protected $reportMapping = array();

    public function beforeProcessReport(DTO\Request\Row $request, DTO\Response $response)
    {
        try {

            $row = $request->getRow();

            $connection = $this->entityManager->getConnection();
            $procedure = new StoredProcedure($connection, 'PEDIDO_COLETA.GETDATAPEDIDOCOLETA');

            $procedure->addParam(new Param('P_CDFILIAL',     Param::PARAM_INPUT, $row['P_CDFILIAL']));
            $procedure->addParam(new Param('P_CDFILIPROC',   Param::PARAM_INPUT, $this->environment->getCdFilial()));
            $procedure->addParam(new Param('P_CDCOMPRADOR',  Param::PARAM_INPUT, $row['P_CDCOMPRADOR']));
            $procedure->addParam(new Param('P_CDPRODINI',    Param::PARAM_INPUT, $row['P_CDPRODINI']));
            $procedure->addParam(new Param('P_CDPRODFIN',    Param::PARAM_INPUT, $row['P_CDPRODFIN']));
            $procedure->addParam(new Param('P_CDFORNECED',   Param::PARAM_INPUT, $row['P_CDFORNECED']));
            $procedure->addParam(new Param('P_CDREGIAO',     Param::PARAM_INPUT, $row['P_CDREGIAO']));
            $procedure->addParam(new Param('P_CDMARCA',      Param::PARAM_INPUT, $row['P_CDMARCA']));
            $procedure->addParam(new Param('P_DTINI',        Param::PARAM_INPUT, $row['P_DTINI']));
            $procedure->addParam(new Param('P_DTFIN',        Param::PARAM_INPUT, $row['P_DTFIN']));
            $procedure->addParam(new Param('P_TIPOPEDIDO',   Param::PARAM_INPUT, $row['P_TIPOPEDIDO']));
            $procedure->addParam(new Param('P_NRCOLEPRECO',  Param::PARAM_INPUT, $row['P_NRCOLEPRECO']));
            $procedure->addParam(new Param('P_DESCFORNCOT',  Param::PARAM_INPUT, $row['P_DESCONSFORN']));
            $procedure->addParam(new Param('P_NRCOMPRA',     Param::PARAM_INPUT, ''));
            $procedure->addParam(new Param('P_CDLEILAO',     Param::PARAM_INPUT, $row['CDLEILAO']));
            $procedure->addParam(new Param('P_NRSEQPRELIST', Param::PARAM_INPUT, $row['NRSEQPRELIST']));

            $procedure->addParam(new Param('P_CDOPERLANC',   Param::PARAM_INPUT, $this->environment->getCdoperador()));
            $procedure->addParam(new Param('P_NRORG',        Param::PARAM_INPUT, $this->environment->getNrOrg()));
            $procedure->addParam(new Param('P_NRPEDCOL',     Param::PARAM_OUTPUT, NULL, Param::PARAM_TYPE_STR, 20));
            $procedure->addParam(new Param('P_ERROR',        Param::PARAM_OUTPUT, NULL, Param::PARAM_TYPE_STR, 20));
            $result = $procedure->execute();

            $row['P_NRPEDCOL'] = $result['P_NRPEDCOL'];
            $row['NRORG'] = $this->environment->getNrOrg();
            $this->qr2Strategy->setQr2Type($row['QR2TYPE']);
            $this->stNomeRel = $this->getReportName($row);
            $this->birtStrategy->setReportFormat(strtolower($row['QR2TYPE']));

            $this->reportMapping[$this->stNomeRel] =
                array(
                    'strategy' => 'BIRT',
                    'parameterMapping' =>
                    array(
                        "P_IMPOBS"          => "P_IMPOBS",
                        "NRORG"             => "P_NRORG",
                        "P_NRPEDCOL"        => "P_NRPEDCOL"

                    )
                );


            if ($row['BOACRESITEM'] == 'S') {
                $this->pedcolPrevcons($row, 'REPORT');
            }
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), 1));
        }
    }

    protected function criarListaValores($lista)
    {
        $nrlista = $this->newCode->getCodeByTableName('LISTA', $this->environment->getNrOrg());
        $valores = explode('_', $lista);
        try {
            foreach ($valores as $key) {
                $this->insertLista($nrlista, $key);
            }
            return $nrlista;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function insertLista($nrlista, $cdlista)
    {
        $params = array(
            "NRLISTA" => $nrlista,
            "CDLISTA" => $cdlista
        );

        $sql = "INSERT INTO LISTA (NRLISTA, CDLISTA) VALUES (:NRLISTA, :CDLISTA)";
        $query = $this->connection->prepare($sql);
        $query->execute($params);
    }

    private function arrayKeyExists($array, $key)
    {
        return array_key_exists($key, $array) && $array && count($array);
    }

    private function concatArray($array, $key)
    {
        return self::arrayKeyExists($array, $key) ? join('_', $array[$key]) : 'T';
    }

    private function strPadRightProdFin($array)
    {
        return self::arrayKeyExists($array, 'CDPRODFIN')
            ? str_pad($array['CDPRODFIN'], 20, 'Z', STR_PAD_RIGHT)
            : str_pad('Z', 20, 'Z', STR_PAD_RIGHT);
    }

    private function setReportValues($row)
    {
        $row['P_NRCOLEPRECO']   = $row['NRCOLEPRECO'];
        $row['P_NRCOMPRA']      = '';
        $row['P_CDREGIAO']      = self::concatArray($row, 'REGIAO');
        $row['P_T_CDREGIAO']    = self::concatArray($row, 'REGIAO');
        $row['P_CDCOMPRADOR']   = self::concatArray($row, 'COMPRADOR');
        $row['P_T_CDCOMPRADOR'] = self::concatArray($row, 'COMPRADOR');
        $row['P_CDFORNECED']    = self::concatArray($row, 'FORNECEDOR');
        $row['P_T_CDFORNECED']  = self::concatArray($row, 'FORNECEDOR');
        $row['P_CDMARCA']       = self::concatArray($row, 'MARCA');
        $row['P_T_CDMARCA']     = self::concatArray($row, 'MARCA');
        $row['P_DTINI']         = $row['INTERVAL_VALIDADE']['start'];
        $row['P_DTFIN']         = $row['INTERVAL_VALIDADE']['end'];
        $row['P_DTINICIAL']     = $row['INTERVAL_VALIDADE']['start'];
        $row['P_DTFINAL']       = $row['INTERVAL_VALIDADE']['end'];
        $row['P_CDPRODINI']     = self::arrayKeyExists($row, 'CDPRODINI') ? $row['CDPRODINI'] : '0';
        $row['P_CDPRODFIN']     = self::strPadRightProdFin($row);
        $row['P_TPRODUTO']      = self::arrayKeyExists($row, 'CDPRODINI') ? ' ' : 'T';
        $row['P_IMPOBS']        = 'N';
        $row['P_DESCONSFORN']   = $row['DESCFORNPRI'];
        $row['P_ACOMPRAR']      = $row['TODOSPROD'];

        return $row;
    }

    public function sendReportToFornecedor(DTO\Request\Row $request, DTO\Response $response)
    {
        $row = self::setReportValues($request->getRow());
        $this->qr2Strategy->setQr2Type($row['QR2TYPE']);
        parent::createRemoteReport(new DTO\Request\Row(new DTO\Row($row), "", "", ""), $response);

        /**
         *
         *
         *
         * To Do
         *
         *
         *
         * envio de email
         *
         * SELECT CDFORNECED, NVL(FORNECEDOR.DSMAILPEDCOLCOP, FORNECEDOR.DSMAILCOTCOP) DSMAILCOTCOP,
         * NVL(
         *  (SELECT DSEMAILFORN FROM TABLE(DADOS_FORN_EMAIL(FORNECEDOR.CDFORNECED, NULL, 'S', :NRORG))),
         *  NVL( FORNECEDOR.DSEMAILPEDCOLFO, FORNECEDOR.DSEMAILCOT ))
         * AS DSEMAILCOT
         * FROM FORNECEDOR
         * WHERE INSTR(:P_CDFORNECED, FORNECEDOR.CDFORNECED) > 0
         *
         *
         *
         */

        $dadosEmail = $rows['email'];

        $messages = [];

        if (is_array($rows['relatorioData'])) {
            for ($i = 0; $i < count($rows['relatorioData']); $i++) {
                try {
                    $row = $rows['relatorioData'][$i];
                    $row['reportLabel'] = $reportParams['NAME'];
                    $row['__report_name'] = $reportParams['NAME'];
                    $this->birtStrategy->setReportFormat($reportParams['TYPE']);
                    parent::createRemoteReport(new DTO\Request\Row(new DTO\Row($reportFilters), "", "", ""), $response);
                    self::enviarEmail($row, $dadosEmail, $response);
                    $this->globalReports = [];
                } catch (\Exception $e) {
                    $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), 1));
                }
            }
        }
    }

    public function cancelDelete(DTO\Request\Row $request, DTO\Response $response)
    {
        $row = $request->getRow();
        $this->service->deletePedidoColetaTmp($row['NRPEDCOL']);
    }
}
