<?php

namespace Services;

use Util\SupUtilities;
use \Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use \Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DTO;

class AprovacaoAlcadaService extends \Teknisa\GER\Controllers\TeknisaCrud
{
    protected $crudService;
    protected $supUtilities;
    protected $environment;
    protected $connection;
    protected $newCode;

    public function __construct($crudService, $parameterBag, SupUtilities $supUtilities, $environment, $connection, $newCode)
    {
        parent::__construct($crudService, $parameterBag);
        $this->crudService  = $crudService;
        $this->supUtilities = $supUtilities;
        $this->environment  = $environment;
        $this->connection   = $connection;
        $this->newCode      = $newCode;
    }

    public function updateWorkflow($row, $status)
    {
        try {
            $query = "UPDATE APROVALCADA 
                         SET IDAPROVADO  = :IDAPROVADO, 
                             DSJUSTIFICA = :DSJUSTIFICA, 
                             DTAPROVACAO = SYSDATE, 
                             CDOPERAPROV = :CDOPERAPROV 
                       WHERE NRAPROVALCADA = :NRAPROVALCADA";
            $params = [
                "NRAPROVALCADA" => $row['NRAPROVALCADA'],
                "IDAPROVADO"    => $status,
                "DSJUSTIFICA"   => isset($row['JUSTIFICATIVA']) ? $row['JUSTIFICATIVA'] : null,
                "CDOPERAPROV"   => $this->environment->getCdoperador(),
            ];
            $this->supUtilities->crud($query, $params);

        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function insertAprovalcadaSQL($row, $nrConfigAlcada, $idTpAlcada, $approver)
    {
        try {
            $sql = "INSERT INTO APROVALCADA(NRAPROVALCADA,
                                            NRCONFIGALCADA,
                                            CDFILDEST,
                                            CDFILLANC,
                                            NRDEMANDA,
                                            CDOPERADOR,
                                            IDTPDEMANDA,
                                            IDCLASSIFIC,
                                            IDOPERACAOALC,
                                            IDAPROVADO,
                                            IDTPALCADA,
                                            DTAPROVACAO,
                                            DTINCLUSAO)
                                    VALUES (:NRAPROVALCADA,
                                            :NRCONFIGALCADA,
                                            :CDFILDEST,
                                            :CDFILLANC,
                                            :NRDEMANDA,
                                            :CDOPERADOR,
                                            :IDTPDEMANDA,
                                            :IDCLASSIFIC,
                                            :IDOPERACAOALC,
                                            :IDAPROVADO,
                                            :IDTPALCADA,
                                            NULL,
                                            SYSDATE)";
            $std = $this->connection->prepare($sql);
            $std->execute(
                array(
                    "NRAPROVALCADA"  => floatval($this->newCode->getCodeByTableName('APROVALCADA', null)),
                    "NRCONFIGALCADA" => $nrConfigAlcada,
                    "CDFILDEST"      => $row['CDFILDEST'],
                    "CDFILLANC"      => $row['CDFILLANC'],
                    "NRDEMANDA"      => $row['NRAUTOFORN'],
                    "CDOPERADOR"     => $approver,
                    "IDTPDEMANDA"    => $row['IDTPDEMANDA'],
                    "IDCLASSIFIC"    => $row['IDCLASSIFIC'],
                    "IDOPERACAOALC"  => $row['IDOPERACAOALC'],
                    "IDAPROVADO"     => $row['IDAPROVADO'],
                    "IDTPALCADA"     => $idTpAlcada,
                )
            );
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function updateOrder($row, $status)
    {
        try {
            $query = "UPDATE AUTOFORNE 
                         SET IDSITAFALCADA  = :SITUACAO,
                             NRCONFIGALCADA = :NRCONFIGALCADA, 
                             IDBAIXAAF      = DECODE(:SITUACAO, 'APROVADA', null, DECODE(:SITUACAO, 'PENDENTE', 3, 5))
                       WHERE CDFILDEST  = :CDFILDEST
                         AND CDFILLANC  = :CDFILLANC
                         AND NRAUTOFORN = :NRAUTOFORN";
            $params = [
                "CDFILDEST"  => $row['CDFILDEST'],
                "CDFILLANC"  => $row['CDFILLANC'],
                "NRAUTOFORN" => $row['NRAUTOFORN'],
                "SITUACAO"   => $status,
                "NRCONFIGALCADA" => $row['NRCONFIGALCADA'],
            ];
            return $this->supUtilities->crud($query, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function getApprover($orderData)
    {
        $query = "SELECT NRALCADAGERFILI, CDOPERADOR, VRAPROVCMP, (SELECT MIN(B.VRAPROVCMP) FROM ALCADAGERFILI B WHERE B.CDFILIAL = ALCADAGERFILI.CDFILIAL AND B.NRCONFIGALCADA = ALCADAGERFILI.NRCONFIGALCADA AND B.VRAPROVCMP > ALCADAGERFILI.VRAPROVCMP) VRAPROVCMPMAX FROM ALCADAGERFILI WHERE CDFILIAL = :P_CDFILIAL AND NRCONFIGALCADA = :P_NRCONFIGALCADA AND VRAPROVCMP = (SELECT MIN(C.VRAPROVCMP) FROM ALCADAGERFILI C WHERE C.CDFILIAL = ALCADAGERFILI.CDFILIAL AND C.NRCONFIGALCADA = ALCADAGERFILI.NRCONFIGALCADA AND ((C.VRAPROVCMP > :P_VRAPROVCMP) OR :P_VRAPROVCMP = 'T')) ORDER BY VRAPROVCMP";
        $params = [
            "P_CDFILIAL" => $orderData['CDFILDEST'],
            "P_VRAPROVCMP" => $orderData['VRAPROVCMP'],
            "P_NRCONFIGALCADA" => $orderData['NRCONFIGALCADA'],
        ];
        return $this->supUtilities->crud($query, $params);
    }

    public function setApprover($userCode, $nrApprove)
    {
        $approvalCode = floatval($this->supUtilities->generateCode('APROVALCADA'));
        $query = "INSERT INTO APROVALCADA ( NRAPROVALCADA, NRCONFIGALCADA, CDFILDEST, CDFILLANC, NRDEMANDA, CDOPERADOR, IDTPDEMANDA, IDCLASSIFIC, IDOPERACAOALC, IDAPROVADO, DTINCLUSAO ) SELECT :P_NEWNRAPROVALCADA, NRCONFIGALCADA, CDFILDEST, CDFILLANC, NRDEMANDA, :P_CDOPERADOR, IDTPDEMANDA, IDCLASSIFIC, IDOPERACAOALC, 'PENDENTE', SYSDATE FROM APROVALCADA WHERE NRAPROVALCADA =:P_NRAPROVALCADA";
        $params = [
            "P_NEWNRAPROVALCADA" => $approvalCode,
            "P_NRAPROVALCADA" => $nrApprove,
            "P_CDOPERADOR" => $userCode,
        ];
        return $this->supUtilities->crud($query, $params);
    }

    public function getOrder($row)
    {
        $query = "SELECT C.NMALCADA, A.NRCONFIGALCADA, C.IDTIPOVRAPROV, A.NRAPROVALCADA, A.IDOPERACAOALC, A.IDCLASSIFIC, A.IDTPDEMANDA, AL.VRAPROVCMP, AL.NRALCADAGERFILI, F.NMFILIAL, AUT.* 
                    FROM AUTOFORNE AUT, APROVALCADA A, ALCADAGERFILI AL, CONFIGALCADA C, FILIAL F
                   WHERE AUT.CDFILDEST = :P_CDFILDEST
                     AND AUT.CDFILLANC = :P_CDFILLANC
                     AND AUT.NRAUTOFORN = :P_NRAUTOFORN
                     AND AUT.NRAUTOFORN = A.NRDEMANDA
                     AND A.NRCONFIGALCADA = AL.NRCONFIGALCADA
                     AND A.NRCONFIGALCADA = C.NRCONFIGALCADA
                     AND AL.NRCONFIGALCADA = C.NRCONFIGALCADA
                     AND A.CDFILDEST = AL.CDFILIAL
                     AND A.CDFILDEST = F.CDFILIAL
                     AND A.CDOPERADOR = AL.CDOPERADOR
                     AND A.CDOPERADOR = :P_APROVADOR";
        $params = [
            "P_CDFILDEST" => $row['CDFILDEST'],
            "P_CDFILLANC" => $row['CDFILLANC'],
            "P_NRAUTOFORN" => $row['NRAUTOFORN'],
            "P_APROVADOR" => $row['APROVADOR'],
        ];
        return $this->supUtilities->crud($query, $params)[0];
    }

    public function getApproversMaxValue($orderData)
    {
        $query = "SELECT MAX(VRAPROVCMP) VALORMAXIMO FROM ALCADAGERFILI WHERE CDFILIAL = :P_CDFILIAL AND NRCONFIGALCADA = :P_NRCONFIGALCADA";
        $params = [
            "P_CDFILIAL" => $orderData['CDFILDEST'],
            "P_NRCONFIGALCADA" => $orderData['NRCONFIGALCADA'],
        ];
        return $this->supUtilities->crud($query, $params)[0];
    }

    public function findProdutoOrcamento($params)
    {
        try {
            $query = "SELECT MAX(PROD.CDARVPROD) CDARVPROD, MAX(IDP.VRORCAMENTO) VRORCAMENTO
                        FROM   ORCAMENTOGOP O
                               INNER JOIN ITEMORCAGOP I
                                       ON I.NRORCAMENTOGOP = O.NRORCAMENTOGOP
                                          AND I.IDTIPOORCAOP IN ( 'CONSUMO', 'DESPENTRADA',
                                                                  'DESPDIVERSA' )
                               INNER JOIN APROVAORCAGOP A
                                       ON A.NRORCAMENTOGOP = O.NRORCAMENTOGOP
                               INNER JOIN ITORCAMESGOP IM
                                       ON IM.NRITEMORCAOP = I.NRITEMORCAOP
                               INNER JOIN ITDETALORCAMESGOP IDP
                                       ON IDP.NRITORCAMESGOP = IM.NRITORCAMESGOP
                               INNER JOIN PRODUTO PROD
                                       ON PROD.CDPRODUTO = IDP.CDPRODUTO
                        WHERE  O.CDFILIAL = :P_CDFILDEST
                               AND O.IDPERIODOORCA = :P_IDPERIODOORCA
                               AND O.NRPERIODOORCA = :P_NRPERIODOORCA
                               AND O.NRANOORCAGOP = TO_CHAR(TO_DATE(:P_DTPROGENTAF), 'YYYY')
                               AND TO_CHAR(IM.DTORCAMENTO, 'MM') = TO_CHAR(TO_DATE(:P_DTPROGENTAF), 'MM')
                               AND IDP.VRORCAMENTO IS NOT NULL
                               AND EXISTS(SELECT 1
                                          FROM   ITEMAUTF,
                                                 PRODUTO
                                          WHERE  ITEMAUTF.CDFILDEST = :P_CDFILDEST
                                                 AND ITEMAUTF.CDFILLANC = :P_CDFILLANC
                                                 AND ITEMAUTF.NRAUTOFORN = :P_NRAUTOFORN
                                                 AND ITEMAUTF.CDPRODUTO = PRODUTO.CDPRODUTO
                                                 AND PRODUTO.CDARVPROD LIKE PROD.CDARVPROD
                                                                            ||'%')";
            $params = [
                "P_IDPERIODOORCA"    => $params['IDPERIODOORCA'],
                "P_NRPERIODOORCA"    => $params['NRPERIODOORCA'],
                "P_DTPROGENTAF"      => $params['DTPROGENTAF'],
                "P_CDFILDEST"        => $params['CDFILDEST'],
                "P_CDFILLANC"        => $params['CDFILLANC'],
                "P_NRAUTOFORN"       => $params['NRDEMANDA'],
            ];

            return $this->supUtilities->crud($query, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function getTotalPeriodoAF($params)
    {
        try {
            $query = "SELECT
                            NVL(SUM(ITEMAUTF.VRPREUNIAF * ITEMAUTF.QTPROGENTAF), 0) VR_TOTAL_AF
                        FROM
                            ITEMAUTF,
                            AUTOFORNE,
                            PRODUTO
                        WHERE
                            AUTOFORNE.CDFILDEST                        = ITEMAUTF.CDFILDEST
                        AND AUTOFORNE.CDFILLANC                      = ITEMAUTF.CDFILLANC
                        AND AUTOFORNE.NRAUTOFORN                     = ITEMAUTF.NRAUTOFORN
                        AND TO_CHAR(ITEMAUTF.DTPROGENTAF, 'MM/YYYY') = TO_CHAR(TO_DATE(:P_DTPROGENTAF),
                            'MM/YYYY')
                        AND NVL(ITEMAUTF.IDBAIXITMAF, '1')    IN ( '1', '2' )
                        AND NVL(AUTOFORNE.IDBAIXAAF, '1') NOT IN ( '3', '4', '5' )
                        AND ITEMAUTF.CDFILDEST                 = :P_CDFILDEST
                        AND ITEMAUTF.NRAUTOFORN               <> :P_NRAUTOFORN
                        AND ITEMAUTF.CDPRODUTO                 = PRODUTO.CDPRODUTO
                        AND PRODUTO.CDARVPROD LIKE :P_CDARVPROD
                            ||'%' ";
            $params = [
                "P_DTPROGENTAF" => $params['DTPROGENTAF'],
                "P_CDFILDEST"   => $params['CDFILDEST'],
                "P_NRAUTOFORN"  => $params['NRDEMANDA'],
                "P_CDARVPROD"   => $params['CDARVPROD'],
            ];

            return $this->supUtilities->crud($query, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function getValorDemanda($params)
    {
        try {
            $query = "SELECT VALOR_DEMANDA(:P_CDFILDEST, :P_NRAUTOFORN, :P_IDPERIODOORCA, :P_NRPERIODOORCA, :P_DTPROGENTAF) VRDEMANDA FROM DUAL";
            $params = [
                "P_DTPROGENTAF"   => $params['DTPROGENTAF'],
                "P_CDFILDEST"     => $params['CDFILDEST'],
                "P_NRAUTOFORN"    => $params['NRDEMANDA'],
                "P_IDPERIODOORCA" => $params['IDPERIODOORCA'],
                "P_NRPERIODOORCA" => $params['NRPERIODOORCA'],
            ];

            return $this->supUtilities->crud($query, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function findDataOrcamento($params)
    {
        try {
            $query = "SELECT V_DATAINI, ADD_MONTHS(V_DATAINI,1)-1 V_DATAFIM 
                        FROM (
                            SELECT TO_DATE(CASE WHEN TO_NUMBER(:P_DIA) > TO_NUMBER(TO_CHAR(LAST_DAY(:P_DATA),'DD'))
                                         THEN TO_CHAR(LAST_DAY(:P_DATA),'DD')
                                         ELSE LPAD(:P_DIA,2,'0')
                                    END || TO_CHAR(P.DTMES,'MM/YYYY'),'DD/MM/YYYY') V_DATAINI
                               FROM DATA P
                              WHERE :P_DATA BETWEEN TO_DATE(CASE WHEN TO_NUMBER(:P_DIA) > TO_NUMBER(TO_CHAR(LAST_DAY(:P_DATA),'DD'))
                                                                THEN TO_CHAR(LAST_DAY(:P_DATA),'DD')
                                                                ELSE LPAD(:P_DIA,2,'0')
                                                           END || TO_CHAR(P.DTMES,'MM/YYYY'),'DD/MM/YYYY')
                                               AND ADD_MONTHS(TO_DATE(CASE WHEN TO_NUMBER(:P_DIA) > TO_NUMBER(TO_CHAR(LAST_DAY(:P_DATA),'DD'))
                                                                           THEN TO_CHAR(LAST_DAY(:P_DATA),'DD')
                                                                           ELSE LPAD(:P_DIA,2,'0')
                                                                      END || TO_CHAR(P.DTMES,'MM/YYYY'),'DD/MM/YYYY'),1)-1
                              GROUP BY TO_DATE(CASE WHEN TO_NUMBER(:P_DIA) > TO_NUMBER(TO_CHAR(LAST_DAY(:P_DATA),'DD'))
                                                 THEN TO_CHAR(LAST_DAY(:P_DATA),'DD')
                                                 ELSE LPAD(:P_DIA,2,'0')
                                            END || TO_CHAR(P.DTMES,'MM/YYYY'),'DD/MM/YYYY'))";
            $params = [
                "P_DIA"       => $params['DIA'],
                "P_DATA"      => $params['DATA'],
            ];

            return $this->supUtilities->crud($query, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function findDiaInicioOrcamento($cdFilialDestino)
    {
        try {
            $query = "SELECT IDCONSAFORCFIL, VRPEORCBLOQFIL, NVL(P.IDVERIORCFILI,'NAO_VALIDA') IDVERIORCFILI,
                            NVL(F.NRDIAINIORCAGOP,PO.NRDIAINIORCA) DIAINIORCA, NVL(PO.CDTIPOCUSTO,'03') CDTIPOCUSTO
                        FROM FILIAL F, PARASUPRI P, PARAMORCAOPERAC PO
                        WHERE F.CDFILIAL = :P_CDFILDEST
                        AND P.CDFILIAL = NVL(F.CDFILCENTCMP, F.CDFILIAL)";
            $params = [
                "P_CDFILDEST" => $cdFilialDestino,
            ];

            return $this->supUtilities->crud($query, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function findPeriodoOrcamento($params)
    {
        try {
            $query = "SELECT MAX(NRPERIODO) V_NRPERIODOORCAGOP, MAX(IDPERIODOORCA) V_IDPERIODOORCA
                         FROM PERIODOORCAGOP P
                        WHERE TO_DATE(:V_DATAINI) BETWEEN TO_DATE(CASE WHEN TO_NUMBER(:DIAINIORCA) > TO_NUMBER(TO_CHAR(LAST_DAY(:V_DATAINI),'DD'))
                                                             THEN TO_CHAR(LAST_DAY(:V_DATAINI),'DD')
                                                             ELSE LPAD(:DIAINIORCA,2,'0')
                                                       END || TO_CHAR(P.DTINIPERIORCA,'MM') || TO_CHAR(TO_DATE(:V_DATAINI), 'YYYY'),'DD/MM/YYYY')
                                            AND ADD_MONTHS(TO_DATE(CASE WHEN TO_NUMBER(:DIAINIORCA) > TO_NUMBER(TO_CHAR(LAST_DAY(:V_DATAINI),'DD'))
                                                                        THEN TO_CHAR(LAST_DAY(:V_DATAINI),'DD')
                                                                        ELSE LPAD(:DIAINIORCA,2,'0')
                                                                   END || TO_CHAR(P.DTFIMPERIORCA,'MM') || TO_CHAR(TO_DATE(:V_DATAINI), 'YYYY'),'DD/MM/YYYY'),1)-1";
            $params = [
                "DIAINIORCA" => $params['DIAINIORCA'],
                "V_DATAINI"  => $params['V_DATAINI'],
            ];

            return $this->supUtilities->crud($query, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function findDataEntregaAF($params)
    {
        try {
            $query = "SELECT DTPROGENTAF
                       FROM ITEMAUTF AF
                      WHERE AF.CDFILDEST  = :P_CDFILDEST
                        AND AF.CDFILLANC  = :P_CDFILLANC
                        AND AF.NRAUTOFORN = :P_NRDEMANDA
                      GROUP BY DTPROGENTAF";
            $params = [
                "P_CDFILDEST" => $params['CDFILDEST'],
                "P_CDFILLANC" => $params['CDFILLANC'],
                "P_NRDEMANDA" => $params['NRDEMANDA'],
            ];

            return $this->supUtilities->crud($query, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function findFilialAlcada($params)
    {
        try {
            $query = "SELECT
                            NRFILIALCADA,
                            NRCONFIGALCADA,
                            CDFILIAL,
                            IDCONSISTVRAPROV
                        FROM
                            FILIALCADA
                        WHERE
                            NRCONFIGALCADA = :P_NRCONFIGALCADA
                            AND CDFILIAL = :P_CDFILDEST";

            $params = [
                "P_CDFILDEST"      => $params['CDFILDEST'],
                "P_NRCONFIGALCADA" => $params['NRCONFIGALCADA'],
            ];

            return $this->supUtilities->crud($query, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function findAprovadorAtual($params)
    {
        try {
            $query = "SELECT
                            AG.NRCONFIGALCADA,
                            AG.CDFILIAL,
                            AG.CDOPERADOR,
                            AG.VRAPROVCMP
                        FROM
                            ALCADAGERFILI   AG
                        WHERE AG.CDOPERADOR = :P_CDOPERADOR
                            AND AG.CDFILIAL = :P_CDFILDEST
                            AND EXISTS (SELECT 1
                                        FROM APROVALCADA A 
                                        WHERE A.NRDEMANDA = :P_NRDEMANDA
                                         AND A.CDOPERADOR = :P_CDOPERADOR
                                         AND A.CDFILDEST = :P_CDFILDEST
                                         AND A.NRCONFIGALCADA = AG.NRCONFIGALCADA)";

            $params = [
                "P_CDFILDEST"  => $params['CDFILDEST'],
                "P_CDOPERADOR" => $params['APROVADOR'],
                "P_NRDEMANDA"  => $params['NRAUTOFORN'],
            ];

            return $this->supUtilities->crud($query, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function executeCONSISTE_ORCAMENTO($unidadeDestino, $unidadeLancamento, $numeroDemanda, $dtEntrega = null, $nrorg = null, $operacao = null)
    {
        try {
            $procedure  = new StoredProcedure($this->connection, 'CONSISTE_ORCAMENTO_GOP');
            $procedure->addParam(new Param('P_CDFILDEST',  Param::PARAM_INPUT,  $unidadeDestino,  Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDFILLANC',  Param::PARAM_INPUT,  $unidadeLancamento,  Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRAUTOFORN', Param::PARAM_INPUT,  $numeroDemanda, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CONSISTE',   Param::PARAM_OUTPUT, NULL, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NROG',       Param::PARAM_INPUT,  $nrorg, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_DATA',       Param::PARAM_INPUT,  $dtEntrega, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_IDOPERACAO', Param::PARAM_INPUT,  $operacao, Param::PARAM_TYPE_STR));

            return $procedure->execute();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function sqlApprover($row)
    {
        return $this->crudService->getFilteredDataSet("checkApprover", $this->criaFiltroAprovador($row))->getRows();
    }

    public function criaFiltroAprovador($row)
    {
        $filter = array(
            array(
                "columnName" => "NRCONFIGALCADA",
                "operator" => "=",
                "value" => $row['NRCONFIGALCADA']
            ),
            array(
                "columnName" => "CDFILIAL",
                "operator" => "=",
                "value" => $row['CDFILDEST']
            ),
            array(
                "columnName" => "CDOPERADOR",
                "operator" => "=",
                "value" => $row['APROVADOR']
            ),
            array(
                "columnName" => "NRDEMANDA",
                "operator" => "=",
                "value" => $row['NRAUTOFORN']
            )
        );
        return $filter;
    }

    public function buscaDIAINIORCA($params)
    {
        try {
            $query = "SELECT IDCONSAFORCFIL, VRPEORCBLOQFIL, 
                        NVL(P.IDVERIORCFILI,'NAO_VALIDA') IDVERIORCFILI,
                        NVL(F.NRDIAINIORCAGOP,PO.NRDIAINIORCA) DIAINIORCA, 
                        NVL(PO.CDTIPOCUSTO,'03') CDTIPOCUSTO,
                        NVL(PO.IDORCDESPDIVERS,'P') IDORCDESPDIVERS, 
                        NVL(PO.IDORCADESPMATRI,'P') IDORCADESPMATRI
                      FROM FILIAL F, PARASUPRI P, PARAMORCAOPERAC PO
                      WHERE F.CDFILIAL = :P_CDFILDEST
                       AND P.CDFILIAL = NVL(F.CDFILCENTCMP, F.CDFILIAL)";

            $params = [
                "P_CDFILDEST"     => $params['CDFILDEST'],
            ];

            return $this->supUtilities->crud($query, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function buscaDTPROGENTAF($params)
    {
        try {
            $query = "SELECT DTPROGENTAF
                       FROM ITEMAUTF AF
                      WHERE AF.NRAUTOFORN = :P_NRDEMANDA
                        AND AF.CDFILDEST  = :P_CDFILDEST
                        AND AF.CDFILLANC  = :P_CDFILLANC
                      GROUP BY DTPROGENTAF";

            $params = [
                "P_NRDEMANDA" => $params['NRAUTOFORN'],
                "P_CDFILDEST" => $params['CDFILDEST'],
                "P_CDFILLANC" => $params['CDFILLANC'],
            ];

            return $this->supUtilities->crud($query, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function buscaDATAINI($params)
    {
        try {
            $query = "SELECT 
                            TO_DATE( :P_DATACASE || TO_CHAR(P.DTMES,'MM/YYYY'),'DD/MM/YYYY') as DATAINI
                    FROM DATA P
                    WHERE :P_DATA BETWEEN 
                            TO_DATE( :P_DATACASE || TO_CHAR(P.DTMES,'MM/YYYY'),'DD/MM/YYYY')
                        AND ADD_MONTHS(
                            TO_DATE( :P_DATACASE || TO_CHAR(P.DTMES,'MM/YYYY'),'DD/MM/YYYY')
                        ,1)-1
                    GROUP BY 
                            TO_DATE( :P_DATACASE || TO_CHAR(P.DTMES,'MM/YYYY'),'DD/MM/YYYY')";

            $params = [
                "P_DATACASE" => $params['DATACASE'],
                "P_DATA"     => $params['DTPROGENTAF'],
            ];

            return $this->supUtilities->crud($query, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }


    public function createDATAFIM($params)
    {
        try {
            $query = "select ADD_MONTHS(:P_DATA,1)-1 as DATAFIM from filial";

            $params = [
                "P_DATA" => $params['DATAINI'],
            ];

            return $this->supUtilities->crud($query, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function buscaPeriodo($params)
    {
        try {
            $query = "SELECT MAX(NRPERIODO) as MAX_NRPERIODO, 
                               MAX(IDPERIODOORCA) as MAX_IDPERIODOORCA
                        FROM PERIODOORCAGOP P
                        WHERE P.CDFILIAL = (SELECT NVL(MAX(PERIODOORCAGOP.CDFILIAL),'P') 
                        FROM PERIODOORCAGOP 
                        WHERE PERIODOORCAGOP.CDFILIAL = :P_CDFILDEST)
                            AND TO_DATE(:P_DATAINI) BETWEEN 
                                                        TO_DATE(CASE WHEN TO_NUMBER(:P_DIAINIORCA) > TO_NUMBER(TO_CHAR(LAST_DAY(TO_DATE(:P_DATAINI)),'DD'))
                                                            THEN TO_CHAR(LAST_DAY(TO_DATE(:P_DATAINI)),'DD')
                                                            ELSE LPAD(:P_DIAINIORCA,2,0)
                                                        END || TO_CHAR(P.DTINIPERIORCA,'MM') || TO_CHAR(TO_DATE(:P_DATAINI), 'YYYY'),'DD/MM/YYYY')
                                                    AND 
                                                        ADD_MONTHS(TO_DATE(CASE WHEN TO_NUMBER(:P_DIAINIORCA) > TO_NUMBER(TO_CHAR(LAST_DAY(TO_DATE(:P_DATAINI)),'DD'))
                                                            THEN TO_CHAR(LAST_DAY(TO_DATE(:P_DATAINI)),'DD')
                                                            ELSE LPAD(:P_DIAINIORCA,2,0)
                                                        END || TO_CHAR(P.DTFIMPERIORCA,'MM') || TO_CHAR(TO_DATE(:P_DATAINI), 'YYYY'),'DD/MM/YYYY'),1)-1";

            $params = [
                "P_CDFILDEST" => $params['CDFILDEST'],
                "P_DATAINI"     => $params['DATAINI'],
                "P_DIAINIORCA"     => $params['DIAINIORCA'],
            ];

            return $this->supUtilities->crud($query, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function buscaOrcamento($params)
    {
        try {
            $query = "SELECT PROD.CDARVPROD, IDP.CDCONTCTBL, IM.DTORCAMENTO, ABS(IM.VRORCAMENTO) VRORCACONSDESP, ABS(IDP.VRORCAMENTO) VRORCAITEM,
                                          I.IDTIPOORCAOP, I.CDDESPPREV, O.IDPERIODOORCA, O.NRPERIODOORCA
                                     FROM ORCAMENTOGOP O INNER JOIN ITEMORCAGOP I ON I.NRORCAMENTOGOP = O.NRORCAMENTOGOP
                                                                                 AND I.IDTIPOORCAOP IN ('CONSUMO','DESPENTRADA','DESPDIVERSA')
                                                         INNER JOIN APROVAORCAGOP A ON A.NRORCAMENTOGOP = O.NRORCAMENTOGOP
                                                                                   AND A.IDTIPOORCAOP = I.IDTIPOORCAOP
                                                         INNER JOIN ITORCAMESGOP IM ON IM.NRITEMORCAOP = I.NRITEMORCAOP
                                                         INNER JOIN ITDETALORCAMESGOP IDP ON IDP.NRITORCAMESGOP = IM.NRITORCAMESGOP
                                                         LEFT  JOIN PRODUTO PROD ON PROD.CDPRODUTO = IDP.CDPRODUTO
                                    WHERE O.CDFILIAL = :P_CDFILDEST
                                      AND O.IDPERIODOORCA = :P_IDPERIODOORCA
                                      AND O.NRPERIODOORCA = :P_NRPERIODOORCAGOP
                                      AND O.NRANOORCAGOP = TO_CHAR(TO_DATE(:P_DATAINI),'YYYY')
                                      AND :P_DTPROGENTAF BETWEEN :P_DATAINI AND :P_DATAFIM
                                      AND TO_NUMBER(TO_CHAR(TO_DATE(:P_DATAINI),'MM')) = TO_NUMBER(TO_CHAR(TO_DATE(IM.DTORCAMENTO),'MM'))
                                      AND IDP.VRORCAMENTO IS NOT NULL
                                      AND EXISTS (SELECT 1
                                                    FROM ITEMAUTF AF INNER JOIN PRODUTO P ON AF.CDPRODUTO = P.CDPRODUTO
                                                   WHERE AF.CDFILDEST  = :P_CDFILDEST
                                                     AND AF.CDFILLANC  = :P_CDFILLANC
                                                     AND AF.NRAUTOFORN = :P_NRDEMANDA
                                                     AND P.CDARVPROD LIKE PROD.CDARVPROD||'%'
                                                     AND PROD.CDARVPROD IS NOT NULL
                                                   UNION
                                                  SELECT 1
                                                    FROM ITEMAUTF AF INNER JOIN PRODUTO P ON AF.CDPRODUTO = P.CDPRODUTO
                                                   WHERE AF.CDFILDEST  = :P_CDFILDEST
                                                     AND AF.CDFILLANC  = :P_CDFILLANC
                                                     AND AF.NRAUTOFORN = :P_NRDEMANDA
                                                     AND CTBL_PRODUTO(AF.CDPRODUTO,'E') LIKE IDP.CDCONTCTBL||'%'
                                                     AND IDP.CDCONTCTBL IS NOT NULL
                                                     AND (NOT EXISTS (SELECT PPR.CDFILIAL
                                                                        FROM PRODUTOROP PPR, PRODUTO PP
                                                                       WHERE PPR.IDTPPRODUTOROP = 'CO'
                                                                         AND PPR.CDFILIAL = TIPOPARAM_DRO(:P_CDFILDEST)
                                                                         AND PPR.CDPRODUTO = PP.CDPRODUTO
                                                                         AND P.CDARVPROD LIKE PP.CDARVPROD||'%')
                                                          OR EXISTS (SELECT PPR.CDFILIAL
                                                                       FROM PRODUTOROP PPR, PRODUTO PP
                                                                      WHERE PPR.IDTPPRODUTOROP = 'OX'
                                                                        AND PPR.CDFILIAL = TIPOPARAM_DRO(:P_CDFILDEST)
                                                                        AND PPR.CDPRODUTO = PP.CDPRODUTO
                                                                        AND P.CDARVPROD LIKE PP.CDARVPROD||'%')))
                                  ORDER BY I.IDTIPOORCAOP, I.CDDESPPREV, PROD.CDARVPROD, IDP.CDCONTCTBL DESC";

            $params = [
                "P_CDFILDEST"        => $params['CDFILDEST'],
                "P_CDFILLANC"        => $params['CDFILLANC'],
                "P_NRDEMANDA"        => $params['NRAUTOFORN'],
                "P_DATAINI"          => $params['DATAINI'],
                "P_DATAFIM"          => $params['DATAFIM'],
                "P_DTPROGENTAF"      => $params['DTPROGENTAF'],
                "P_IDPERIODOORCA"    => $params['IDPERIODOORCA'],
                "P_NRPERIODOORCAGOP" => $params['NRPERIODO'],
            ];
            return $this->supUtilities->crud($query, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function getOperador()
    {
        return $this->environment->getCdoperador();
    }

    public function getConsisteOrcamento($row)
    {
        try {
            $query = "SELECT NRCONFIGALCADA, IDTIPORCAOP, CDDESPPREV, VRORCAMENTO, IDTPALCADA
                        FROM (SELECT CONFIGALCADA.NRCONFIGALCADA, CONFIGALCADA.IDTIPORCAOP, NVL(CONFIGALCADA.CDDESPPREV, 'T') CDDESPPREV, CONSISTORCAMENTO.VRORCAMENTO, CONFIGALCADA.IDTPALCADA
                                FROM CONSISTORCAMENTO INNER JOIN AUTOFORNE ON CONSISTORCAMENTO.CDFILDEST = AUTOFORNE.CDFILDEST
                                                                          AND CONSISTORCAMENTO.CDFILLANC = AUTOFORNE.CDFILLANC
                                                                          AND CONSISTORCAMENTO.NRDEMANDA = AUTOFORNE.NRAUTOFORN
                                                      INNER JOIN CONFIGALCADA ON CONSISTORCAMENTO.IDORIGEMCONS = CONFIGALCADA.IDTIPORCAOP
                                                                             AND NVL(CONSISTORCAMENTO.CDDESPPREV, 'XXX') = NVL(CONFIGALCADA.CDDESPPREV, 'XXX')
                               WHERE AUTOFORNE.CDFILDEST = :CDFILDEST
                                 AND AUTOFORNE.CDFILLANC = :CDFILLANC
                                 AND AUTOFORNE.NRAUTOFORN = :NRAUTOFORN
                                 AND AUTOFORNE.IDSITUATUAAF IS NULL
                                 AND AUTOFORNE.IDAFFORALIM = 'N'
                                 AND CONFIGALCADA.IDLANCAFEXTRA = :IDLANCAFEXTRA
                                 AND CONFIGALCADA.IDTIPOAF = :IDTIPOAF
                                 AND EXISTS (SELECT 1
                                               FROM ALCADAGERFILI
                                              WHERE ALCADAGERFILI.CDFILIAL = AUTOFORNE.CDFILDEST
                                                AND ALCADAGERFILI.NRCONFIGALCADA = CONFIGALCADA.NRCONFIGALCADA)
                               ORDER BY CONSISTORCAMENTO.NRCONSISTORCAMENTO DESC)
                               WHERE ROWNUM = 1";

            $params = [
                "CDFILDEST"     => $row['CDFILDEST'],
                "CDFILLANC"     => $row['CDFILLANC'],
                "NRAUTOFORN"    => $row['NRAUTOFORN'],
                "IDLANCAFEXTRA" => $row['IDLANCAFEXTRA'],
                "IDTIPOAF"      => $row['IDTIPOAF'],
            ];

            return $this->supUtilities->crud($query, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function getConsisteOrca($row) {
        try {
            $query = "SELECT NRCONSISTORCAMENTO, IDORIGEMCONS, CDDESPPREV
                        FROM (SELECT NRCONSISTORCAMENTO, CONSISTORCAMENTO.IDORIGEMCONS, DESPPREV.CDDESPPREV ||' - '|| DESPPREV.NMDESPPREV AS CDDESPPREV
                                FROM DESPPREV LEFT JOIN (SELECT NRCONSISTORCAMENTO, IDORIGEMCONS, CDDESPPREV
                                                           FROM CONSISTORCAMENTO
                                                          WHERE CDFILDEST = :CDFILDEST
                                                            AND CDFILLANC = :CDFILLANC
                                                            AND NRDEMANDA = :NRAUTOFORN
                                                          ORDER BY NRCONSISTORCAMENTO DESC) CONSISTORCAMENTO 
                                                     ON DESPPREV.CDDESPPREV = CONSISTORCAMENTO.CDDESPPREV
                               WHERE CONSISTORCAMENTO.IDORIGEMCONS = 'DESPESA' 
                               UNION 
                              SELECT NRCONSISTORCAMENTO, IDORIGEMCONS, CDDESPPREV
                                FROM (SELECT NRCONSISTORCAMENTO, IDORIGEMCONS, CDDESPPREV
                                        FROM CONSISTORCAMENTO
                                       WHERE CDFILDEST = :CDFILDEST
                                         AND CDFILLANC = :CDFILLANC
                                         AND NRDEMANDA = :NRAUTOFORN
                                         AND IDORIGEMCONS = 'CONSUMO' 
                                       ORDER BY NRCONSISTORCAMENTO DESC))                       
                       WHERE ROWNUM = 1";

            $params = [
                "CDFILDEST"     => $row['CDFILDEST'],
                "CDFILLANC"     => $row['CDFILLANC'],
                "NRAUTOFORN"    => $row['NRAUTOFORN'],                
            ];

            return $this->supUtilities->crud($query, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function getFirstApprover($row, $data)
    {
        try {
            $sql = "SELECT NRCONFIGALCADA, CDFILIAL, CDOPERADOR, VRAPROVCMP
                      FROM (SELECT NRCONFIGALCADA, CDFILIAL, CDOPERADOR, VRAPROVCMP
                              FROM ALCADAGERFILI
                             WHERE NRCONFIGALCADA = :NRCONFIGALCADA
                               AND CDFILIAL = :CDFILIAL
                             ORDER BY VRAPROVCMP)
                     WHERE ROWNUM = 1";
            $std = $this->connection->prepare($sql);
            $std->execute(
                array(
                    "CDFILIAL"       => $row['CDFILDEST'],
                    "NRCONFIGALCADA" => $data['NRCONFIGALCADA']
                )
            );
            $result = $std->fetchAll();
            return isset($result) && count($result) ? end($result)['CDOPERADOR'] : null;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function deleteAprovalcada($row, $data)
    {
        try {
            $sql = "DELETE FROM APROVALCADA 
                     WHERE NRCONFIGALCADA = :NRCONFIGALCADA
                       AND CDFILDEST = :CDFILDEST
                       AND NRDEMANDA = :NRDEMANDA";
            $std = $this->connection->prepare($sql);
            $std->execute(
                array(
                    'NRCONFIGALCADA' => $data['NRCONFIGALCADA'],
                    'CDFILDEST' => $row['CDFILDEST'],
                    'NRDEMANDA' => $row['NRAUTOFORN'],
                )
            );
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function getEmailOper($cdfildest, $nrConfigAlcada, $oper)
    {
        try {
            $sql = "SELECT OPERADOR.CDOPERADOR,
                           OPERADOR.NMOPERADOR,
                           OPERADOR.DSEMAILOPER
                      FROM OPERADOR
                     WHERE OPERADOR.CDOPERADOR = :CDOPERADOR
                     UNION   
                    SELECT OPERADOR.CDOPERADOR,
                           OPERADOR.NMOPERADOR,
                           OPERADOR.DSEMAILOPER
                      FROM ALCADAGEROPER INNER JOIN OPERADOR ON ALCADAGEROPER.CDOPERRELACAP = OPERADOR.CDOPERADOR
                     WHERE ALCADAGEROPER.NRCONFIGALCADA = :NRCONFIGALCADA
                       AND ALCADAGEROPER.CDFILIAL = :CDFILIAL
                       AND ALCADAGEROPER.CDOPERADOR = :CDOPERADOR";
            $std = $this->connection->prepare($sql);
            $std->execute(
                array(
                    "CDOPERADOR" => $oper,
                    "NRCONFIGALCADA" => $nrConfigAlcada,
                    "CDFILIAL" => $cdfildest,
                )
            );
            return $std->fetchAll();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function getNextApprover($row, $totalOrderValue)
    {
        try {
            $sql = "SELECT NRALCADAGERFILI, NRCONFIGALCADA, CDFILIAL, CDOPERADOR, VRAPROVCMP
                      FROM (SELECT NRALCADAGERFILI, NRCONFIGALCADA, CDFILIAL, CDOPERADOR, VRAPROVCMP
                              FROM ALCADAGERFILI
                             WHERE NRCONFIGALCADA = :NRCONFIGALCADA
                               AND CDFILIAL = :CDFILIAL
                               AND CDOPERADOR != :CDOPERADOR
                               AND VRAPROVCMP > :VRAPROVCMP
                               AND EXISTS (SELECT 1
                                             FROM ALCADAGERFILI AG
                                            WHERE AG.NRCONFIGALCADA = ALCADAGERFILI.NRCONFIGALCADA
                                              AND AG.CDFILIAL = ALCADAGERFILI.CDFILIAL
                                              AND AG.CDOPERADOR = :CDOPERADOR
                                              AND AG.VRAPROVCMP < :VRAPROVCMP)
                             ORDER BY VRAPROVCMP)
                      WHERE ROWNUM = 1";
            $std = $this->connection->prepare($sql);
            $std->execute(
                array(
                    "CDFILIAL"       => $row['CDFILDEST'],
                    "NRCONFIGALCADA" => $row['NRCONFIGALCADA'],
                    "CDOPERADOR"     => $row['APROVADOR'],
                    "VRAPROVCMP"     => $totalOrderValue,
                )
            );
            return $std->fetchAll();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function deleteAllTmpItemautfDel($row)
    {
        try {
            $query = "DELETE TMP_ITEMAUTFDEL                         
                       WHERE CDFILDEST  = :CDFILDEST
                         AND CDFILLANC  = :CDFILLANC
                         AND NRAUTOFORN = :NRAUTOFORN";
            $params = [
                "CDFILDEST"  => $row['CDFILDEST'],
                "CDFILLANC"  => $row['CDFILLANC'],
                "NRAUTOFORN" => $row['NRAUTOFORN'],
            ];
            return $this->supUtilities->crud($query, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function getTotalAF($row)
    {
        try {
            $query = "SELECT SUM((NVL(ITEMAUTF.VRPREUNIAF,0) * NVL(ITEMAUTF.QTPROGENTAF,0))) VRTOTAL
                        FROM ITEMAUTF
                       WHERE CDFILDEST = :CDFILDEST
                         AND CDFILLANC = :CDFILLANC
                         AND NRAUTOFORN = :NRAUTOFORN";
            
            $std = $this->connection->prepare($query);
            $std->execute(
                array(
                    "CDFILDEST"  => $row['CDFILDEST'],
                    "CDFILLANC"  => $row['CDFILLANC'],
                    "NRAUTOFORN" => $row['NRAUTOFORN'] ?? $row['NRDEMANDA'],
                )
            );
            $totalAF = $std->fetchAll();
            if(isset($totalAF) && count($totalAF)) {
                return end($totalAF)['VRTOTAL'];
            }

            return 0;
        } catch (\Exception $e) {
            throw $e;
        }
    }
}
